import { __decorate } from "tslib";
import { Extension } from '@roots/bud-framework/extension';
import { bind, label } from '@roots/bud-framework/extension/decorators';
import { Config } from './config.js';
/**
 * Babel support for `@roots/bud`
 *
 * @public
 */
let BabelExtension = class BabelExtension extends Extension {
    /**
     * Babel cache directory
     *
     * @public
     */
    get cacheDirectory() {
        return this.app.path(`@storage/cache/babel`);
    }
    /**
     * Babel env
     *
     * @public
     */
    get env() {
        return {
            development: {
                compact: false,
            },
        };
    }
    /**
     * Root directory
     *
     * @public
     */
    get root() {
        return this.app.path();
    }
    /**
     * Babel RuleSetItem callback
     *
     * @public
     * @decorator `@bind`
     */
    setRuleSetItem(ruleSetItem) {
        return ruleSetItem.setLoader('babel').setOptions(() => ({
            cacheDirectory: this.cacheDirectory,
            presets: Object.values(this.app.babel.presets),
            plugins: Object.values(this.app.babel.plugins),
            env: this.env,
            root: this.root,
        }));
    }
    /**
     * Initialize extension
     *
     * @public
     * @decorator `@bind`
     */
    async init() {
        this.app.babel = new Config();
    }
    /**
     * Register extension
     *
     * @public
     * @decorator `@bind`
     */
    async register() {
        this.app.babel
            .setPresets({
            '@babel/preset-env': await this.resolve('@babel/preset-env'),
        })
            .setPlugins({
            '@babel/plugin-transform-runtime': [
                await this.resolve('@babel/plugin-transform-runtime'),
                { helpers: false },
            ],
            '@babel/plugin-proposal-object-rest-spread': await this.resolve('@babel/plugin-proposal-object-rest-spread'),
            '@babel/plugin-syntax-dynamic-import': await this.resolve('@babel/plugin-syntax-dynamic-import'),
            '@babel/plugin-proposal-class-properties': await this.resolve('@babel/plugin-proposal-class-properties'),
        });
        this.app.build
            .setLoader('babel', await this.resolve('babel-loader'))
            .setItem('babel', this.setRuleSetItem);
        this.app.build.rules.js.setUse(items => ['babel', ...items]);
    }
};
__decorate([
    bind
], BabelExtension.prototype, "setRuleSetItem", null);
__decorate([
    bind
], BabelExtension.prototype, "init", null);
__decorate([
    bind
], BabelExtension.prototype, "register", null);
BabelExtension = __decorate([
    label('@roots/bud-babel')
], BabelExtension);
export default BabelExtension;
//# sourceMappingURL=extension.js.map